""" Addon Information """
# ------------
# EventScripts Script-Addon:    - BackRaw's AfterBan (BAB, bab.py, 8.2kb)
# Author:                       - BackRaw - http://forums.eventscripts.com/profile.php?mode=viewprofile&u=13683
#                                           http://addons.eventscripts.com/addons/user/13683
# Version:                      - 1.1
# ------------

""" Imports """
# ------------
# EventScripts Core-Module       - ES functionality
import es
# cPickle                        - Database-Management
import cPickle
# Popuplib's Easymenu            - Popup menu
from popuplib import easymenu
# os.path's isfile               - Check file-existent
from os.path import isfile
# Config Object                  - Configuration file: ./addons/eventscripts/bab/bab_admins.ini
from configobj import ConfigObj
# ------------

""" Global Variables """
# ------------
# This dictionary will hold the banned players and their ban-state.
players = {}
# We'll need the addon's path more often...
path = es.getAddonPath('bab')
# We'll need the addon's prefix more often...
prefix = '#green[#lightgreenAfterBan#green]#default'
# ------------

""" Class - UseridError """
# ------------
# This class creates an exception especially for UserIDs
# ------------
class UseridError(Exception):
    pass

""" Class - DB """
# ------------
# This will manage the database-file: ./addons/eventscripts/bab/bab_database.db
# ------------
class DB(object):
    def __init__(self):
        self.database = path + '/bab_database.db'
    def loadDatabase(self):
        if isfile(self.database):
            f = open(self.database, 'rb')
            db = cPickle.load(f)
            f.close()
            return db
        return {}
    def saveDatabase(self):
        f = open(self.database, 'wb')
        cPickle.dump(players, f)
        f.close()
db = DB()

""" Class - AfterBan """
# ------------
# This class is the actual un-/ban-management
# ------------
class AfterBan(object):
    def __init__(self, userid):
        self.userid = userid
    def isAdmin(self):
        return es.getplayersteamid(self.userid) in ConfigObj(path + '/bab_admins.ini')['admins']
    def banPlayer(self, argc, steamid, force):
        global menu
        if self.isAdmin():
            if argc == 1:
                deletePopup()
                menu = easymenu('bab_banned_playersPopup', '_popup_choice', menuSelect)
                menu.settitle('Choose a player to Un-/Ban:')
                for player in players:
                    user = players[player]
                    menu.addoption(player, '%s - Banned: %s' % (user['name'], user['banned']))
                menu.send(self.userid)
            elif argc == 2:
                self.userid = steamid
                self.banManagement()
            elif argc == 3:
                if force.isdigit():
                    force = int(force)
                    if force in (1, 0):
                        self.userid = steamid
                        self.banManagement(force)
                    else:
                        es.tell(self.userid, '#multi', prefix, 'Wrong State (must be 1 or 0): #lightgreen!ab <SteamID> <1/0>')
                else:
                    es.tell(self.userid, '#multi', prefix, 'Wrong State (must be numerical): #lightgreen!ab <SteamID> <1/0>')
            else:
                es.tell(self.userid, '#multi', prefix, 'Wrong Syntax - either: #lightgreen!ab #defaultor #lightgreen!ab <SteamID> #defaultor #lightgreen!ab <SteamID> <1/0 (ban-state)>')
        else:
            es.tell(self.userid, '#multi', prefix, 'You are not authorized as an Admin!')
    def banManagement(self, force='no'):
        if str(self.userid).isdigit():
            self.userid = es.getplayersteamid(self.userid)
        if force == 'no':
            if players[self.userid]['banned']:
                players[self.userid]['banned'] = 0
            else:
                players[self.userid]['banned'] = 1
        else:
            players[self.userid]['banned'] = force
        user = players[self.userid]
        es.msg('#multi', prefix, 'Player #green%s #defaulthas been %s the banned-list.' % (user['name'], 'added to' if user['banned'] else 'removed from'))
        es.server.queuecmd('%s;writeid' % ('banid 0 %s kick' % self.userid if user['banned'] else 'removeid %s' % self.userid))
    def listBanned(self):
        if self.isAdmin():
            es.cexec(self.userid, 'echo ')
            es.cexec(self.userid, 'echo ----------------------------------------------')
            es.cexec(self.userid, 'echo AfterBan - Listing Banned Players...')
            es.cexec(self.userid, 'echo ')
            for player in players:
                es.cexec(self.userid, 'echo %s - %s' % (player, players[player]['name']))
            es.cexec(self.userid, 'echo ')
        else:
            es.tell(self.userid, '#multi', prefix, 'You are not authorized as an Admin!')

def afterBan(userid):
    if es.exists('userid', userid):
        return AfterBan(userid)
    raise UseridError, 'Userid %s does not exist!' % userid

""" Menu-Handling """
# ------------
# Popup-deleting management
# ------------
def deletePopup():
    try:
        menu.delete()
    except:
        pass

# ------------
# List the banned players to the client's console - Say-Command "!list"
# ------------
def listbanned():
    afterBan(es.getcmduserid()).listBanned()

# ------------
# Say-Command "!ab"
# ------------
def ban():
    argc = es.getargc()
    afterBan(es.getcmduserid()).banPlayer(argc, es.getargv(1) if argc >= 1 else '', es.getargv(2) if argc >= 2 else '')

# ------------
# Menu-Select for the popup
# ------------
def menuSelect(userid, choice, popupid):
    afterBan(es.getuserid(choice)).banManagement()
    user = players[choice]
    menu.setoption(choice, '%s - Banned: %s' % (user['name'], user['banned']))
    menu.send(userid)

""" Events """
# ------------
# Either add the player to the dictionary or change their name to their present one on player_activate
# ------------
def player_activate(ev):
    steam = ev['es_steamid']
    name = ev['es_username']
    if not afterBan(ev['userid']).isAdmin():
        if steam in players:
            players[steam]['name'] = name
        else:
            players[steam] = {'banned':0, 'name':name}
    db.saveDatabase()

# ------------
# Save the database whenever a player disconnects
# ------------
def player_disconnect(ev):
    db.saveDatabase()

""" Events - Mani (and other) compatibility """
# ------------
# Both of these events only act either if the player is not or is saved in the dictionary, depending on the event
# ------------

# ------------
# This will add the banned player to the dictionary and set their ban-state to 1 and add the current name
# ------------
def server_addban(ev):
    steam = ev['networkid']
    if steam not in players:
        players[steam] = {'banned':1, 'name':ev['name']}
        db.saveDatabase()

# ------------
# This will add the banned player to the dictionary and set their ban-state to 0 and add their current name
# ------------
def server_removeban(ev):
    steam = ev['networkid']
    if steam in players:
        players[steam] = {'banned':0, 'name':es.getplayername(es.getuserid(steam))}
        db.saveDatabase()

""" EventScripts main-control functions """
# ------------
# On load add all players to the dictionary if they're not already in it and create the Say-Commands "!ab" and "!list"
# ------------
def load():
    es.ServerVar('bab_ver', '1.1').makepublic()
    for userid in filter(lambda x: not afterBan(x).isAdmin() and es.getplayersteamid(x) not in players, es.getUseridList()):
        players[es.getplayersteamid(userid)] = {'banned':0, 'name':es.getplayername(userid)}
    for x in ('!ab', '!list'):
        if not es.exists('saycommand', x):
            es.regsaycmd(x, 'bab/%s' % x.replace('!ab', 'ban').replace('!list', 'listbanned'))

# ------------
# On unload, delete the popups, un-register the Say-Commands "!ab" and "!list" and save the database
# ------------
def unload():
    deletePopup()
    for x in ('!ab', '!list'):
        es.unregsaycmd(x)
    db.saveDatabase()

""" End Code """
